@@ -1,6 +1,6 @@
 // gold.cc -- main linker functions
 
-// Copyright 2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+// Copyright 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
 // Written by Ian Lance Taylor <iant@google.com>.
 
 // This file is part of gold.
@@ -30,6 +30,7 @@
 #include "libiberty.h"
 
 #include "options.h"
+#include "target-select.h"
 #include "debug.h"
 #include "workqueue.h"
 #include "dirsearch.h"
@@ -48,18 +49,25 @@
 namespace gold
 {
 
+class Object;
+
 const char* program_name;
 
+static Task*
+process_incremental_input(Incremental_binary*, unsigned int, Input_objects*,
+			  Symbol_table*, Layout*, Dirsearch*, Mapfile*,
+			  Task_token*, Task_token*);
+
 void
-gold_exit(bool status)
+gold_exit(Exit_status status)
 {
   if (parameters != NULL
       && parameters->options_valid()
       && parameters->options().has_plugins())
     parameters->options().plugins()->cleanup();
-  if (!status && parameters != NULL && parameters->options_valid())
+  if (status != GOLD_OK && parameters != NULL && parameters->options_valid())
     unlink_if_ordinary(parameters->options().output_file_name());
-  exit(status ? EXIT_SUCCESS : EXIT_FAILURE);
+  exit(status);
 }
 
 void
@@ -80,7 +88,7 @@ gold_nomem()
       const char* const s = ": out of memory\n";
       len = write(2, s, strlen(s));
     }
-  gold_exit(false);
+  gold_exit(GOLD_ERR);
 }
 
 // Handle an unreachable case.
@@ -90,7 +98,7 @@ do_gold_unreachable(const char* filename, int lineno, const char* function)
 {
   fprintf(stderr, _("%s: internal error in %s, at %s:%d\n"),
 	  program_name, function, filename, lineno);
-  gold_exit(false);
+  gold_exit(GOLD_ERR);
 }
 
 // This class arranges to run the functions done in the middle of the
@@ -168,8 +176,16 @@ queue_initial_tasks(const General_options& options,
 {
   if (cmdline.begin() == cmdline.end())
     {
+      bool is_ok = false;
       if (options.printed_version())
-	gold_exit(true);
+	is_ok = true;
+      if (options.print_output_format())
+	{
+	  print_output_format();
+	  is_ok = true;
+	}
+      if (is_ok)
+	gold_exit(GOLD_OK);
       gold_fatal(_("no input files"));
     }
 
@@ -178,19 +194,32 @@ queue_initial_tasks(const General_options& options,
     thread_count = cmdline.number_of_input_files();
   workqueue->set_thread_count(thread_count);
 
-  if (parameters->incremental())
+  // For incremental links, the base output file.
+  Incremental_binary* ibase = NULL;
+
+  if (parameters->incremental_update())
     {
-      Incremental_checker incremental_checker(
-          parameters->options().output_file_name(),
-          layout->incremental_inputs());
-      if (incremental_checker.can_incrementally_link_output_file())
-        {
-          // TODO: remove when incremental linking implemented.
-          printf("Incremental linking might be possible "
-              "(not implemented yet)\n");
-        }
-      // TODO: If we decide on an incremental build, fewer tasks
-      // should be scheduled.
+      Output_file* of = new Output_file(options.output_file_name());
+      if (of->open_base_file(options.incremental_base(), true))
+	{
+	  ibase = open_incremental_binary(of);
+	  if (ibase != NULL
+	      && ibase->check_inputs(cmdline, layout->incremental_inputs()))
+	    ibase->init_layout(layout);
+	  else
+	    {
+	      delete ibase;
+	      ibase = NULL;
+	      of->close();
+	    }
+	}
+      if (ibase == NULL)
+	{
+	  if (set_parameters_incremental_full())
+	    gold_info(_("linking with --incremental-full"));
+	  else
+	    gold_fallback(_("restart link with --incremental-full"));
+	}
     }
 
   // Read the input files.  We have to add the symbols to the symbol
@@ -198,16 +227,45 @@ queue_initial_tasks(const General_options& options,
   // each input file.  We associate the blocker with the following
   // input file, to give us a convenient place to delete it.
   Task_token* this_blocker = NULL;
-  for (Command_line::const_iterator p = cmdline.begin();
-       p != cmdline.end();
-       ++p)
+  if (ibase == NULL)
     {
-      Task_token* next_blocker = new Task_token(true);
-      next_blocker->add_blocker();
-      workqueue->queue(new Read_symbols(input_objects, symtab, layout,
-					&search_path, 0, mapfile, &*p, NULL,
-					NULL, this_blocker, next_blocker));
-      this_blocker = next_blocker;
+      // Normal link.  Queue a Read_symbols task for each input file
+      // on the command line.
+      for (Command_line::const_iterator p = cmdline.begin();
+	   p != cmdline.end();
+	   ++p)
+	{
+	  Task_token* next_blocker = new Task_token(true);
+	  next_blocker->add_blocker();
+	  workqueue->queue(new Read_symbols(input_objects, symtab, layout,
+					    &search_path, 0, mapfile, &*p, NULL,
+					    NULL, this_blocker, next_blocker));
+	  this_blocker = next_blocker;
+	}
+    }
+  else
+    {
+      // Incremental update link.  Process the list of input files
+      // stored in the base file, and queue a task for each file:
+      // a Read_symbols task for a changed file, and an Add_symbols task
+      // for an unchanged file.  We need to mark all the space used by
+      // unchanged files before we can start any tasks running.
+      unsigned int input_file_count = ibase->input_file_count();
+      std::vector<Task*> tasks;
+      tasks.reserve(input_file_count);
+      for (unsigned int i = 0; i < input_file_count; ++i)
+	{
+	  Task_token* next_blocker = new Task_token(true);
+	  next_blocker->add_blocker();
+	  Task* t = process_incremental_input(ibase, i, input_objects, symtab,
+					      layout, &search_path, mapfile,
+					      this_blocker, next_blocker);
+	  tasks.push_back(t);
+	  this_blocker = next_blocker;
+	}
+      // Now we can queue the tasks.
+      for (unsigned int i = 0; i < tasks.size(); i++)
+	workqueue->queue(tasks[i]);
     }
 
   if (options.has_plugins())
@@ -220,13 +278,11 @@ queue_initial_tasks(const General_options& options,
       this_blocker = next_blocker;
     }
 
-  if (parameters->options().relocatable()
-      && (parameters->options().gc_sections()
-	  || parameters->options().icf_enabled()))
+  if (options.relocatable()
+      && (options.gc_sections() || options.icf_enabled()))
     gold_error(_("cannot mix -r with --gc-sections or --icf"));
 
-  if (parameters->options().gc_sections()
-      || parameters->options().icf_enabled())
+  if (options.gc_sections() || options.icf_enabled())
     {
       workqueue->queue(new Task_function(new Gc_runner(options,
 						       input_objects,
@@ -248,6 +304,129 @@ queue_initial_tasks(const General_options& options,
     }
 }
 
+// Process an incremental input file: if it is unchanged from the previous
+// link, return a task to add its symbols from the base file's incremental
+// info; if it has changed, return a normal Read_symbols task.  We create a
+// task for every input file, if only to report the file for rebuilding the
+// incremental info.
+
+static Task*
+process_incremental_input(Incremental_binary* ibase,
+			  unsigned int input_file_index,
+			  Input_objects* input_objects,
+			  Symbol_table* symtab,
+			  Layout* layout,
+			  Dirsearch* search_path,
+			  Mapfile* mapfile,
+			  Task_token* this_blocker,
+			  Task_token* next_blocker)
+{
+  const Incremental_binary::Input_reader* input_reader =
+      ibase->get_input_reader(input_file_index);
+  Incremental_input_type input_type = input_reader->type();
+
+  // Get the input argument corresponding to this input file, matching on
+  // the argument serial number.  If the input file cannot be matched
+  // to an existing input argument, synthesize a new one.
+  const Input_argument* input_argument =
+      ibase->get_input_argument(input_file_index);
+  if (input_argument == NULL)
+    {
+      Input_file_argument file(input_reader->filename(),
+			       Input_file_argument::INPUT_FILE_TYPE_FILE,
+			       "", false, parameters->options());
+      Input_argument* arg = new Input_argument(file);
+      arg->set_script_info(ibase->get_script_info(input_file_index));
+      input_argument = arg;
+    }
+
+  gold_debug(DEBUG_INCREMENTAL, "Incremental object: %s, type %d",
+	     input_reader->filename(), input_type);
+
+  if (input_type == INCREMENTAL_INPUT_SCRIPT)
+    {
+      // Incremental_binary::check_inputs should have cancelled the
+      // incremental update if the script has changed.
+      gold_assert(!ibase->file_has_changed(input_file_index));
+      return new Check_script(layout, ibase, input_file_index, input_reader,
+			      this_blocker, next_blocker);
+    }
+
+  if (input_type == INCREMENTAL_INPUT_ARCHIVE)
+    {
+      Incremental_library* lib = ibase->get_library(input_file_index);
+      gold_assert(lib != NULL);
+      if (lib->filename() == "/group/"
+	  || !ibase->file_has_changed(input_file_index))
+	{
+	  // Queue a task to check that no references have been added to any
+	  // of the library's unused symbols.
+	  return new Check_library(symtab, layout, ibase, input_file_index,
+				   input_reader, this_blocker, next_blocker);
+	}
+      else
+	{
+	  // Queue a Read_symbols task to process the archive normally.
+	  return new Read_symbols(input_objects, symtab, layout, search_path,
+				  0, mapfile, input_argument, NULL, NULL,
+				  this_blocker, next_blocker);
+	}
+    }
+
+  if (input_type == INCREMENTAL_INPUT_ARCHIVE_MEMBER)
+    {
+      // For archive members, check the timestamp of the containing archive.
+      Incremental_library* lib = ibase->get_library(input_file_index);
+      gold_assert(lib != NULL);
+      // Process members of a --start-lib/--end-lib group as normal objects.
+      if (lib->filename() != "/group/")
+	{
+	  if (ibase->file_has_changed(lib->input_file_index()))
+	    {
+	      return new Read_member(input_objects, symtab, layout, mapfile,
+				     input_reader, this_blocker, next_blocker);
+	    }
+	  else
+	    {
+	      // The previous contributions from this file will be kept.
+	      // Mark the pieces of output sections contributed by this
+	      // object.
+	      ibase->reserve_layout(input_file_index);
+	      Object* obj = make_sized_incremental_object(ibase,
+							  input_file_index,
+							  input_type,
+							  input_reader);
+	      return new Add_symbols(input_objects, symtab, layout,
+				     search_path, 0, mapfile, input_argument,
+				     obj, lib, NULL, this_blocker,
+				     next_blocker);
+	    }
+	}
+    }
+
+  // Normal object file or shared library.  Check if the file has changed
+  // since the last incremental link.
+  if (ibase->file_has_changed(input_file_index))
+    {
+      return new Read_symbols(input_objects, symtab, layout, search_path, 0,
+			      mapfile, input_argument, NULL, NULL,
+			      this_blocker, next_blocker);
+    }
+  else
+    {
+      // The previous contributions from this file will be kept.
+      // Mark the pieces of output sections contributed by this object.
+      ibase->reserve_layout(input_file_index);
+      Object* obj = make_sized_incremental_object(ibase,
+						  input_file_index,
+						  input_type,
+						  input_reader);
+      return new Add_symbols(input_objects, symtab, layout, search_path, 0,
+			     mapfile, input_argument, obj, NULL, NULL,
+			     this_blocker, next_blocker);
+    }
+}
+
 // Queue up a set of tasks to be done before queueing the middle set
 // of tasks.  This is only necessary when garbage collection
 // (--gc-sections) of unused sections is desired.  The relocs are read
@@ -317,11 +496,7 @@ queue_middle_tasks(const General_options& options,
   if (parameters->options().gc_sections())
     {
       // Find the start symbol if any.
-      Symbol* start_sym;
-      if (parameters->options().entry())
-        start_sym = symtab->lookup(parameters->options().entry());
-      else
-        start_sym = symtab->lookup("_start");
+      Symbol* start_sym = symtab->lookup(parameters->entry());
       if (start_sym != NULL)
         {
           bool is_ordinary;
@@ -359,6 +534,7 @@ queue_middle_tasks(const General_options& options,
            p != input_objects->relobj_end();
            ++p)
         {
+          Task_lock_obj<Object> tlo(task, *p);
           (*p)->layout(symtab, layout, NULL);
         }
     }
@@ -395,7 +571,8 @@ queue_middle_tasks(const General_options& options,
   // generate an empty file.  Existing builds depend on being able to
   // pass an empty archive to the linker and get an empty object file
   // out.  In order to do this we need to use a default target.
-  if (input_objects->number_of_input_objects() == 0)
+  if (input_objects->number_of_input_objects() == 0
+      && layout->incremental_base() == NULL)
     parameters_force_valid_target();
 
   int thread_count = options.thread_count_middle();
@@ -440,6 +617,15 @@ queue_middle_tasks(const General_options& options,
 	}
     }
 
+  // For incremental updates, record the existing GOT and PLT entries,
+  // and the COPY relocations.
+  if (parameters->incremental_update())
+    {
+      Incremental_binary* ibase = layout->incremental_base();
+      ibase->process_got_plt(symtab, layout);
+      ibase->emit_copy_relocs(symtab);
+    }
+
   if (is_debugging_enabled(DEBUG_SCRIPT))
     layout->script_options()->print(stderr);
 
@@ -557,7 +743,7 @@ queue_middle_tasks(const General_options& options,
 	  // THIS_BLOCKER to be NULL here.  There's no real point in
 	  // continuing if that happens.
 	  gold_assert(parameters->errors()->error_count() > 0);
-	  gold_exit(false);
+	  gold_exit(GOLD_ERR);
 	}
     }
 
